#!/bin/bash
# ******************************************************************
#
# Creates a set of Zones for use with TREP.
#
#
# Before you attempt to run this script you should customize
# the configuration files it uses to match your requirement.
#
# It is mandatory to configure the following files before
# you run this script:
#
#   config/general.cfg
#          This file controls general script behaviour.
#          See the comments in the file for more information.
#
#   config/trep-hosts.cfg
#          This file contains a list of the zones you want to 
#          create. See the comments in the file for more 
#          information.
#
#   config/ips-packages.cfg
#          This file contains a list of additional IPS packages
#          to be installed into each non-global zone after
#          it has been created.
#          See the comments in the file for more information.
#
#
# The following are template files. You'll normally not 
# change this.
#
#   templates/zonecfg.txt
#          This file forms input to the 'zonecfg' command while
#          the script creates the zone. Variable ${HOSTNAME}
#          is allowed as substition variable within the file.
#
#   templates/trep-host-template.xml
#          This file forms input to the 'sysconfig' config command.
#          See comments in the file for more information.
#
#   templates/root_profile
#          This is the Bash .profile file that the root user in
#          the zone will be given. 
#
#   templates/radmin_profile
#          This is the Bash .profile file that the radmin user in
#          the zone will be given. 
#
#   templates/etc_services
#          The contents of this file is added to '/etc/services'
#          in each zone.
#
# ------------------------------------------------------------------
#
# Usage:
#   The scripts does not take any parameters. The script doesn't 
#   support relative invocation, meaning invocation *must* happen
#   from the directory where it lives.
#   
#   The script should be executed from root account or by using
#   'sudo'.
#
# ------------------------------------------------------------------
#
# Requirements:
#
#   -  Solaris 11.0 or later
#
# ------------------------------------------------------------------
#
# Addicticks, 2014
#
# ******************************************************************


echo " "
echo "Zone creation script for TREP"
echo " "
echo " "


cleanup="yes"


# Utility function to replace substitution variables
# in a given file.
replace_var() {
  #   $1 : variable name - in the file this must be 
  #          on the form :   ${VARNAME}
  #
  #   $2 : substitution text
  #
  #   $3 : input file

  sed -e 's/\${'"$1"'}/'"$2"'/g' $3 > /tmp/$$_sed.tmp
  mv /tmp/$$_sed.tmp $3

}


#
# Function to await a zone to boot fully
#
await_boot_multiuser() {
  # $1 = zonename

  sleep 20
  while [ -z "$(zlogin ${1} svcs svc:/milestone/multi-user | grep online)" ]; do
    sleep 10
    echo "   Awaiting zone ${1} to have booted to multi-user state. Have patience ..."
  done
  echo "   Zone ${1} was booted to multi-user state"
}



# Source the general settings
. config/general.cfg
# Timezone var might have '/' in it so we need to escape
# such chars.
Z_TIMEZONE=`echo $Z_TIMEZONE | sed 's/\//\\\\\//g'`







# Loop over existing zones and get rid of them

if [ $cleanup == "yes" ]; then
  for i in `zoneadm list -c`; do
    if [ $i != "global" ]; then
      echo "Getting rid of zone $i"
      zoneadm -z $i halt &>/dev/null
      zoneadm -z $i uninstall -F &>/dev/null
      zonecfg -z $i delete -F
    fi
  done
  
  
  if [ -d "/export/zones" ]; then
    zfs destroy -r rpool/export/zones
  fi
  
  
  dladm delete-etherstub bbswitch0 &>/dev/null
fi





# Create etherstub (think of it as a switch) to be used
# for the TREP backbone net. Basically this will be used
# for inter-node multicast traffic.

echo "Creating etherstub : bbswitch0 ..."
dladm create-etherstub bbswitch0
# For some reason the default MTU for an etherstub is 9000
# but this value doesn't work well with virtualizers such
# as VMware and VirtualBox. So set the MTU back to the 
# standard value.
dladm set-linkprop -p mtu=1500 bbswitch0





# ************************
# Zone loop
# ************************

echo "Creating zones ..."
# Loop over the zones we want to create
for i in `grep -v "^#" config/trep-hosts.cfg`; do 
  zonename=`echo "$i" | awk -F';' '{print $1}'`
  ipaddr=`echo "$i" | awk -F';' '{print $2}'`
  defrouter=`echo "$i" | awk -F';' '{print $3}'`
  ipaddr_bb=`echo "$i" | awk -F';' '{print $4}'`

  echo "Processing zone \"$zonename\" with frontend ip $ipaddr and backbone ip $ipaddr_bb"




  echo "   Configuring zone ..."
  grep -v "^#" templates/zonecfg.txt > /tmp/zonecfg.txt
  replace_var HOSTNAME "$zonename" /tmp/zonecfg.txt
 
  zonecfg -z $zonename -f /tmp/zonecfg.txt
  rm /tmp/zonecfg.txt






  
  echo "   Installing zone ... (this may take a while)"

  cp templates/trep-host-template.xml /tmp/trep-host-template.xml
  
  replace_var Z_TIMEZONE "$Z_TIMEZONE" /tmp/trep-host-template.xml
  replace_var HOSTNAME "$zonename" /tmp/trep-host-template.xml
  replace_var Z_CONSOLE_KEYBOARD_LAYOUT "$Z_CONSOLE_KEYBOARD_LAYOUT" /tmp/trep-host-template.xml
  replace_var STATIC_IP_ADDRESS "$ipaddr" /tmp/trep-host-template.xml
  replace_var DEFAULT_ROUTER "$defrouter" /tmp/trep-host-template.xml
  replace_var DNS_SERVER "$Z_DNS_SERVER" /tmp/trep-host-template.xml
  
#    cat /tmp/trep-host-template.xml
  
#    zoneadm -z $zonename install -c /tmp/trep-host-template.xml &>/dev/null
  zoneadm -z $zonename install -c /tmp/trep-host-template.xml
  rm /tmp/trep-host-template.xml




  echo "   Configuring hosts file for the zone ... "
  hostsfile="/export/zones/$zonename/root/etc/inet/hosts"
  rm $hostsfile
  echo "#" >> $hostsfile
  echo "# Copyright 2009 Sun Microsystems, Inc.  All rights reserved." >> $hostsfile
  echo "# Use is subject to license terms." >> $hostsfile
  echo "#" >> $hostsfile
  echo "# Internet host table" >> $hostsfile
  echo "#" >> $hostsfile
  echo "::1 localhost" >> $hostsfile
  echo "127.0.0.1 localhost loghost" >> $hostsfile
  echo "" >> $hostsfile
  echo "$ipaddr   $zonename   $zonename.trepinabox.org" >> $hostsfile
  echo "$ipaddr_bb   ${zonename}-bb" >> $hostsfile
  for ii in `grep -v "^#" config/trep-hosts.cfg`; do 
  other_zonename=`echo "$ii" | awk -F';' '{print $1}'`
  other_ipaddr=`echo "$ii" | awk -F';' '{print $2}'`
  other_ipaddr_bb=`echo "$ii" | awk -F';' '{print $4}'`
    if [ "$zonename" != "$other_zonename" ]; then
      echo "$other_ipaddr   $other_zonename" >> $hostsfile
      echo "$other_ipaddr_bb   ${other_zonename}-bb" >> $hostsfile
    fi
  done
  chmod +r $hostsfile






  echo "   Booting zone ... (this may take a while)"
  zoneadm -z $zonename boot
  await_boot_multiuser $zonename






  echo "   Install extra IPS packages into the zone ... "
  pkglist=$(grep -v "^#" config/ips-packages.cfg | grep -v "^[ \t]*$")
  for xPKGNAME in $pkglist; do
     echo "      --- installing $xPKGNAME" 
     zlogin $zonename pkg install --accept $xPKGNAME &>/dev/null
  done


  echo "   Configuring bb interface address in the zone ... "
  zlogin $zonename ipadm create-ip bb0
  zlogin $zonename ipadm create-addr -T static -a ${ipaddr_bb}/24 bb0/v4



  echo "   Configuring Bash login profiles for the zone ... "
  cp templates/root_profile /export/zones/$zonename/root/root/.profile
  cp templates/radmin_profile /export/zones/$zonename/root/export/home/radmin/.profile
  zlogin $zonename chown radmin:staff /export/home/radmin/.profile



  echo "   Configuring persistent route for multicast for the zone ... "
  zlogin $zonename route -p add $Z_BACKBONE_MC_ADDRESS_RANGE -gateway  $ipaddr_bb -interface



  echo "   Adding TREP specific ports to the /etc/services file in the zone ... "
  cat templates/etc_services >> /export/zones/$zonename/root/etc/inet/services


  echo "   Adding Solaris Auth descriptor (auth_attr) for TREP mgmt in the zone ... "
  echo "com.thomsonreuters.smf.manage.trep:::TREP daemon management::" >> /export/zones/$zonename/root/etc/security/auth_attr
  zlogin $zonename usermod -S files -A com.thomsonreuters.smf.manage.trep radmin


done



